#include <iostream>
#include <cassert>
#include <cstdint>
#include <clocale>  //  setlocale
using namespace std;

//       0  15   
char nibble_to_hex(uint8_t i) {
    //    
    assert(0x0 <= i && i <= 0xf);

    //     
    static const char digits[] = "0123456789abcdef";
    return digits[i];
}

//       ('0'  '1')
char bit_digit(uint8_t byte, uint8_t bit) {
    return (byte & (0x1 << bit)) ? '1' : '0';
}

//   void*  uint8_t*
const uint8_t* as_bytes(const void* data) {
    return reinterpret_cast<const uint8_t*>(data);
}

//      
void print_in_hex(uint8_t byte) {
    //   
    cout << nibble_to_hex(byte >> 4);
    //   
    cout << nibble_to_hex(byte & 0xf);
}

//      
void print_in_hex(const void* data, size_t size) {
    const uint8_t* bytes = as_bytes(data);
    for (size_t i = 0; i < size; i++) {
        print_in_hex(bytes[i]);

        //  :        16 
        if ((i + 1) % 16 == 0) {
            cout << '\n';
        } else {
            cout << ' ';
        }
    }
}

//      
void print_in_binary(uint8_t byte) {
    //   8 ,   
    for (int bit = 7; bit >= 0; bit--) {
        cout << bit_digit(byte, bit);
    }
}

//      
void print_in_binary(const void* data, size_t size) {
    const uint8_t* bytes = as_bytes(data);
    for (size_t i = 0; i < size; i++) {
        print_in_binary(bytes[i]);

        //  :        4 
        if ((i + 1) % 4 == 0) {
            cout << '\n';
        } else {
            cout << ' ';
        }
    }
}

//   
int main() {
    //     
    setlocale(LC_ALL, "Russian");

    //   nibble_to_hex
    assert(nibble_to_hex(0x0) == '0');
    assert(nibble_to_hex(0xa) == 'a');
    assert(nibble_to_hex(0xf) == 'f');

    //    
    uint32_t test = 0x42;
    cout << " :\n";
    print_in_hex(&test, sizeof(test));
    cout << "\n\n :\n";
    print_in_binary(&test, sizeof(test));
    cout << '\n';

    return 0;
}
